<?php
namespace Concrete\Package\CkeditorKaomoji;

use Concrete\Core\Package\Package;
use Concrete\Core\Permission\Checker as PermissionChecker;
use Concrete\Core\Asset\AssetList;
use Concrete\Core\Asset\Asset;
use Concrete\Core\View\View;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Kaomoji\Kaomoji;
use Concrete\Core\Page\Page;

class Controller extends Package
{
    protected $pkgHandle = 'ckeditor_kaomoji';
    protected $appVersionRequired = '8.0';
    protected $pkgVersion = '2.0';
    protected $pkg;
    protected $permissions;
    protected $settings;

    protected $pkgAutoloaderRegistries = [
        'src/Kaomoji' => '\Kaomoji',
    ];

    public function getPackageDescription()
    {
        return t("Insert hundreds of Japanese Emojis in your content %s Developed by Nour Akalay @ %sKALMOYA - bespoke Concrete CMS development%s", '<br /><span style="font-size:11px;">', '<a target="_blank" href="https://www.kalmoya.com">', '</a></span>');
    }

    public function getPackageName()
    {
        return t("Kaomoji for CKEditor");
    }

    public function on_start()
    {
        $al = AssetList::getInstance();
        $al->register(
            'css',
            'kaomoji/kaomoji-styling',
            'css/kaomoji-styles.css',
            ['version' => '1.0',
                'position' => Asset::ASSET_POSITION_HEADER,
                'minify' => true,
                'combine' => true, ],
            $this
        );

        // Event dispatcher
        /** @var EventDispatcherInterface $event */
        $event = $this->app->make(EventDispatcherInterface::class);

        $event->addListener(
            'on_start',
            function ($event) {
                $page = Page::getCurrentPage();
                if (is_object($page) && !$page->isError()) {
                    if (!$page->isAdminArea()) {
                        $permissions = new PermissionChecker($page);
                        if ($permissions->canEditPageContents()) {
                            app()->make(Kaomoji::class)->ActivatePlugins();
                        }
                    }
                }
            }
        );

        $event->addListener('on_page_view', function () {
            $v = View::getInstance();
            if (is_object($v)) {
                $v->requireAsset('css', 'kaomoji/kaomoji-styling');
            }
        });
    }

    public function install()
    {
        $this->pkg = parent::install();
    }

    public function upgrade()
    {
        parent::upgrade();
    }
}
