/**
 * @license Copyright (c) 2017, Kalmoya - Nour Akalay. All rights reserved.
 * For licensing, see LICENSE.TXT or http://www.concrete5.org/help/legal/commercial_add-on_license
 */

CKEDITOR.dialog.add('kaomoji', function(editor) {
    // Simulate "this" of a dialog for non-dialog events.
    // @type {CKEDITOR.dialog}
    var dialog,
        lang = editor.lang.kaomoji,
        kaomojisSelectGroup = editor.config.kaomojiSymbols,
        kaomojisGroup = [],
        kaomojiGroupSymbols = [],
        ii = 0,
        selectFirstChoice;

    for (kaomojiCategory in lang) {
        if (typeof kaomojisSelectGroup[kaomojiCategory] !== 'undefined') {
            selectFirstChoice = ii ? selectFirstChoice : kaomojiCategory;
            kaomojisGroup.push([lang[kaomojiCategory], kaomojiCategory]);
            kaomojiGroupSymbols.push([lang[kaomojiCategory], kaomojisSelectGroup[kaomojiCategory], kaomojiCategory]);
            ii++;
        }
    }

    var onChoice = function(evt) {
        var target, value;
        if (evt.data)
            target = evt.data.getTarget();
        else
            target = new CKEDITOR.dom.element(evt);

        if (target.getName() == 'a' && (value = target.getChild(0).getHtml())) {
            target.removeClass('cke_light_background');
            dialog.hide();

            // We must use "insertText" here to keep text styled.
            var em = editor.document.createElement('em'),
                span = editor.document.createElement('span');
            span.setHtml('&nbsp;');
            em.setHtml(value);
            em.addClass('cked-kaomoji');
            em.setAttribute('title', target.getAttribute('data-category'));
            editor.insertText(span.getText());
            editor.insertElement(em);
            editor.insertText(span.getText());
        }
    };

    var onClick = CKEDITOR.tools.addFunction(onChoice);

    var focusedNode;

    var onFocus = function(evt, target) {
        var value;
        target = target || evt.data.getTarget();

        if (target.getName() == 'span')
            target = target.getParent();

        if (target.getName() == 'a' && (value = target.getChild(0).getHtml())) {
            // Trigger blur manually if there is focused node.
            if (focusedNode)
                onBlur(null, focusedNode);

            dialog.getContentElement('info', 'kaomojiPreview').getElement().setHtml(value);
            // htmlPreview.setHtml( CKEDITOR.tools.htmlEncode( value ) );
            target.getParent().addClass('cke_light_background');

            // Memorize focused node.
            focusedNode = target;
        }
    };

    var onBlur = function(evt, target) {
        target = target || evt.data.getTarget();

        if (target.getName() == 'span')
            target = target.getParent();

        if (target.getName() == 'a') {
            dialog.getContentElement('info', 'kaomojiPreview').getElement().setHtml('&nbsp;');
            // dialog.getContentElement( 'info', 'htmlPreview' ).getElement().setHtml( '&nbsp;' );
            target.getParent().removeClass('cke_light_background');

            focusedNode = undefined;
        }
    };

    var onKeydown = CKEDITOR.tools.addFunction(function(ev) {
        ev = new CKEDITOR.dom.event(ev);

        // Get an Anchor element.
        var element = ev.getTarget();
        var relative, nodeToMove;
        var keystroke = ev.getKeystroke(),
            rtl = editor.lang.dir == 'rtl';

        switch (keystroke) {
            // UP-ARROW
            case 38:
                // relative is TR
                if ((relative = element.getParent().getParent().getPrevious())) {
                    nodeToMove = relative.getChild([element.getParent().getIndex(), 0]);
                    nodeToMove.focus();
                    onBlur(null, element);
                    onFocus(null, nodeToMove);
                }
                ev.preventDefault();
                break;
                // DOWN-ARROW
            case 40:
                // relative is TR
                if ((relative = element.getParent().getParent().getNext())) {
                    nodeToMove = relative.getChild([element.getParent().getIndex(), 0]);
                    if (nodeToMove && nodeToMove.type == 1) {
                        nodeToMove.focus();
                        onBlur(null, element);
                        onFocus(null, nodeToMove);
                    }
                }
                ev.preventDefault();
                break;
                // SPACE
                // ENTER is already handled as onClick
            case 32:
                onChoice({ data: ev });
                ev.preventDefault();
                break;

                // RIGHT-ARROW
            case rtl ? 37:
                39:
                    // relative is TD
                    if ((relative = element.getParent().getNext())) {
                        nodeToMove = relative.getChild(0);
                        if (nodeToMove.type == 1) {
                            nodeToMove.focus();
                            onBlur(null, element);
                            onFocus(null, nodeToMove);
                            ev.preventDefault(true);
                        } else {
                            onBlur(null, element);
                        }
                    }
                    // relative is TR
                    else
                if ((relative = element.getParent().getParent().getNext())) {
                    nodeToMove = relative.getChild([0, 0]);
                    if (nodeToMove && nodeToMove.type == 1) {
                        nodeToMove.focus();
                        onBlur(null, element);
                        onFocus(null, nodeToMove);
                        ev.preventDefault(true);
                    } else {
                        onBlur(null, element);
                    }
                }
                break;

                // LEFT-ARROW
            case rtl ? 39:
                37:
                    // relative is TD
                    if ((relative = element.getParent().getPrevious())) {
                        nodeToMove = relative.getChild(0);
                        nodeToMove.focus();
                        onBlur(null, element);
                        onFocus(null, nodeToMove);
                        ev.preventDefault(true);
                    }
                    // relative is TR
                    else
                if ((relative = element.getParent().getParent().getPrevious())) {
                    nodeToMove = relative.getLast().getChild(0);
                    nodeToMove.focus();
                    onBlur(null, element);
                    onFocus(null, nodeToMove);
                    ev.preventDefault(true);
                } else {
                    onBlur(null, element);
                }
                break;
            default:
                // Do not stop not handled events.
                return;
        }
    });

    return {
        title: lang.title,
        minWidth: 480,
        minHeight: 280,
        buttons: [CKEDITOR.dialog.cancelButton],
        kaomojiColumns: 5,
        onLoad: function() {
            var columns = this.definition.kaomojiColumns,
                kaomojiGroups = kaomojiGroupSymbols, //editor.config.kaomojiSymbols,
                j = 0,
                html = [],
                kaomojis, kaomojisTableLabel, html, j, kaomoji, kaomojiDesc;

            for (var i = 0; i < kaomojiGroups.length; i++) {
                kaomojisTableLabel = CKEDITOR.tools.getNextId() + '_kaomoji_table_label';
                html.push('<table id="' + kaomojiGroups[i][2] + '" role="listbox" class="kaomoji-group-table" aria-labelledby="' + kaomojisTableLabel + '"' +
                    ' style="width: 100%; height: 100%; border-collapse: separate;"' +
                    ' align="center" cellspacing="2" cellpadding="2" border="0">' +
                    '<thead><tr><td colspan="5">' +
                    kaomojiGroups[i][0] + '</td></tr></thead><tbody>');

                j = 0;
                kaomojis = kaomojiGroups[i][1];
                size = kaomojis.length;

                while (j < size) {
                    html.push('<tr role="presentation">');

                    for (var k = 0; k < columns; k++, j++) {

                        if ((kaomoji = kaomojis[j])) {
                            kaomojiDesc = '';

                            if (kaomoji instanceof Array) {
                                kaomojiDesc = kaomoji[1];
                                kaomoji = kaomoji[0];
                            } else {
                                var _tmpName = kaomoji;

                                // Use kaomoji in case description unavailable.
                                kaomojiDesc = kaomoji;
                            }

                            var kaomojiLabelId = 'cke_kaomoji_label_' + j + '_' + CKEDITOR.tools.getNextNumber();

                            html.push('<td class="cke_dark_background" style="cursor: default; width: 20%;" role="presentation">' + '<a href="javascript: void(0);" role="option"' + ' data-category="' + CKEDITOR.tools.htmlEncode(kaomojiGroups[i][0]) + '" aria-posinset="' + (j + 1) + '"', ' aria-setsize="' + size + '"', ' aria-labelledby="' + kaomojiLabelId + '"', ' class="cke_kaomoji" title="', CKEDITOR.tools.htmlEncode(kaomojiDesc), '"' + ' onkeydown="CKEDITOR.tools.callFunction( ' + onKeydown + ', event, this )"' + ' onclick="CKEDITOR.tools.callFunction(' + onClick + ', this); return false;"' + ' tabindex="-1">' + '<span style="margin: 0 auto;cursor: inherit; font-family: Verdana, Geneva, sans-serif;">' + kaomoji + "</span>" + '<span class="cke_voice_label" id="' + kaomojiLabelId + '">' + kaomojiDesc + "</span></a>");
                        } else {
                            html.push('<td class="cke_dark_background">&nbsp;');
                        }

                        html.push('</td>');
                    }
                    html.push('</tr>');
                }
                html.push('</tbody></table>', '<span id="' + kaomojisTableLabel + '" class="cke_voice_label">' + lang.options + '</span>');
            }

            this.getContentElement('info', 'kaomojiContainer').getElement().setHtml(html.join(''));
        },
        contents: [{
            id: 'info',
            label: editor.lang.common.generalTab,
            title: editor.lang.common.generalTab,
            padding: 0,
            align: 'top',
            elements: [{
                    id: 'kaomojiCategory',
                    type: 'select',
                    className: 'cke_kaomoji_select',
                    style: 'margin-left: auto; margin-right: auto;',
                    label: lang.kaomojiTypeSelector,
                    items: kaomojisGroup,
                    default: selectFirstChoice,
                    onShow: function(event) {
                        var kaomojiContainerElement = dialog.getContentElement('info', 'kaomojiContainer').getElement(),
                            currentKaomojiActive = kaomojiContainerElement.findOne('.active'),
                            firstSelectedKaomojiGroup = kaomojiContainerElement.getElementsByTag('table').getItem(0);
                        if (currentKaomojiActive) {
                            currentKaomojiActive.removeClass('active');
                        }
                        if (firstSelectedKaomojiGroup) {
                            firstSelectedKaomojiGroup.addClass('active');
                        }

                    },
                    onChange: function() {
                        var value = this.getValue();
                        if (typeof value === 'string') {
                            if (value && value.length > 0) {
                                var kaomojiContainerElement = dialog.getContentElement('info', 'kaomojiContainer').getElement(),
                                    // kaomojiGroup = kaomojiContainerElement.getElementsByTag('table'),
                                    currentKaomojiActive = kaomojiContainerElement.findOne('.active'),
                                    firstSelectedKaomojiGroup = CKEDITOR.document.getById(value),
                                    firstSelectedKaomoji = firstSelectedKaomojiGroup.getElementsByTag('a').getItem(0);

                                if (currentKaomojiActive) {
                                    currentKaomojiActive.removeClass('active');
                                }
                                if (firstSelectedKaomojiGroup) {
                                    firstSelectedKaomojiGroup.addClass('active');
                                }
                                if (firstSelectedKaomoji) {
                                    firstSelectedKaomoji.focus();
                                    onFocus(null, firstSelectedKaomoji);
                                }

                            }
                        }
                    }
                },
                {
                    type: 'html',
                    id: 'kaomojiPreview',
                    className: 'cke_dark_background',
                    style: 'border:1px solid #eeeeee;font-size:20px;height:40px;width:100%;padding-top:9px;font-family:Verdana, Geneva, sans-serif;text-align:center;',
                    html: '<div>&nbsp;</div>'
                },
                {
                    type: 'html',
                    id: 'kaomojiContainer',
                    html: '',
                    onMouseover: onFocus,
                    onMouseout: onBlur,
                    focus: function() {
                        var firstKaomoji = this.getElement().getElementsByTag('a').getItem(0);
                        setTimeout(function() {
                            firstKaomoji.focus();
                            onFocus(null, firstKaomoji);
                        }, 0);
                    },
                    onShow: function() {
                        var el = this.getElement();
                        var firstKaomoji = this.getElement().getElementsByTag('a').getItem(0); //.getChild( [ 0, 1, 0, 0, 0 ] );
                        setTimeout(function() {
                            firstKaomoji.focus();
                            onFocus(null, firstKaomoji);
                        }, 0);
                    },
                    onLoad: function(event) {
                        dialog = event.sender;
                    }
                }
            ]
        }]
    };
});